Package: GenomicInteractions
Type: Package
Title: Utilities for handling genomic interaction data
URL:
        https://github.com/ComputationalRegulatoryGenomicsICL/GenomicInteractions/
Version: 1.47.0
Date: 2025-07-22
Author: Harmston, N., Ing-Simmons, E., Perry, M., Baresic, A., Lenhard,
        B.
Maintainer: Liz Ing-Simmons <liz.ingsimmons@gmail.com>
Imports: Rsamtools, rtracklayer, GenomicRanges (>= 1.29.6), IRanges,
        BiocGenerics (>= 0.15.3), data.table, stringr, Seqinfo,
        ggplot2, grid, gridExtra, methods, igraph, S4Vectors (>=
        0.13.13), dplyr, Gviz, Biobase, graphics, stats, utils,
        grDevices
Suggests: knitr, rmarkdown, BiocStyle, testthat
VignetteBuilder: knitr
Description: Utilities for handling genomic interaction data such as
        ChIA-PET or Hi-C, annotating genomic features with interaction
        information, and producing plots and summary statistics.
biocViews: Software,Infrastructure,DataImport,DataRepresentation,HiC
License: GPL-3
Depends: R (>= 3.5), InteractionSet
RoxygenNote: 7.1.2
Config/pak/sysreqs: cmake libglpk-dev make libbz2-dev libicu-dev
        libjpeg-dev liblzma-dev libpng-dev libuv1-dev libxml2-dev
        libssl-dev xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:39:48 UTC
RemoteUrl: https://github.com/bioc/GenomicInteractions
RemoteRef: HEAD
RemoteSha: 921dc32e02340056b669bb0793c2135a025851b8
NeedsCompilation: no
Packaged: 2026-05-30 08:13:18 UTC; root
Built: R 4.6.0; ; 2026-05-30 11:48:11 UTC; unix
