# Generated by roxygen2: do not edit by hand

export("interactionCounts<-")
export("name<-")
export(GenomicInteractions)
export(InteractionTrack)
export(anchorOne)
export(anchorTwo)
export(annotateInteractions)
export(annotateRegions)
export(annotationFeatures)
export(availableDisplayPars)
export(calculateDistances)
export(categoriseInteractions)
export(countsBetweenAnchors)
export(export.bed12)
export(export.bedpe)
export(export.chiasig)
export(export.igraph)
export(get_binom_ligation_threshold)
export(get_self_ligation_threshold)
export(interactionCounts)
export(is.cis)
export(is.dd)
export(is.dt)
export(is.pd)
export(is.pp)
export(is.pt)
export(is.trans)
export(is.tt)
export(isInteractionType)
export(makeGenomicInteractionsFromFile)
export(name)
export(plotAvgViewpoint)
export(plotCisTrans)
export(plotCounts)
export(plotDists)
export(plotInteractionAnnotations)
export(plotSummaryStats)
export(plotViewpoint)
export(removeDups)
export(resetAnnotations)
export(subsetByFeatures)
export(summariseByFeaturePairs)
export(summariseByFeatures)
export(viewPoint)
exportClasses(GenomicInteractions)
exportMethods("description<-")
exportMethods("interactionCounts<-")
exportMethods("name<-")
exportMethods(anchorOne)
exportMethods(anchorTwo)
exportMethods(annotateInteractions)
exportMethods(annotateRegions)
exportMethods(annotationFeatures)
exportMethods(asBED)
exportMethods(calculateDistances)
exportMethods(chromosome)
exportMethods(countsBetweenAnchors)
exportMethods(description)
exportMethods(end)
exportMethods(export.bed12)
exportMethods(export.bedpe)
exportMethods(export.chiasig)
exportMethods(export.igraph)
exportMethods(interactionCounts)
exportMethods(is.cis)
exportMethods(is.dd)
exportMethods(is.dt)
exportMethods(is.pd)
exportMethods(is.pp)
exportMethods(is.pt)
exportMethods(is.trans)
exportMethods(is.tt)
exportMethods(isInteractionType)
exportMethods(name)
exportMethods(resetAnnotations)
exportMethods(start)
exportMethods(subset)
exportMethods(subsetByFeatures)
exportMethods(sum)
exportMethods(summariseByFeaturePairs)
exportMethods(summariseByFeatures)
exportMethods(updateObject)
import(GenomicRanges)
import(Gviz)
import(InteractionSet)
import(ggplot2)
import(grid)
import(methods)
importFrom(Biobase,updateObject)
importFrom(BiocGenerics,getObjectSlots)
importFrom(BiocGenerics,relist)
importFrom(BiocGenerics,which)
importFrom(Gviz,availableDisplayPars)
importFrom(IRanges,IRanges)
importFrom(IRanges,PartitioningByWidth)
importFrom(IRanges,overlapsAny)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,bamFlagAsBitMatrix)
importFrom(Rsamtools,scanBam)
importFrom(Rsamtools,scanBamFlag)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,elementNROWS)
importFrom(S4Vectors,first)
importFrom(S4Vectors,make_zero_col_DFrame)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,runValue)
importFrom(S4Vectors,second)
importFrom(S4Vectors,setValidity2)
importFrom(S4Vectors,subjectHits)
importFrom(Seqinfo,'seqlevels<-')
importFrom(Seqinfo,genome)
importFrom(Seqinfo,isCircular)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(data.table,.N)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(dplyr,group_by_)
importFrom(dplyr,mutate_)
importFrom(dplyr,n)
importFrom(dplyr,summarise_)
importFrom(grDevices,col2rgb)
importFrom(graphics,plot)
importFrom(gridExtra,marrangeGrob)
importFrom(igraph,graph_from_data_frame)
importFrom(rtracklayer,asBED)
importFrom(rtracklayer,export)
importFrom(stats,binom.test)
importFrom(stats,complete.cases)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,sd)
importFrom(stringr,str_match)
importFrom(stringr,str_split)
importFrom(utils,read.table)
importFrom(utils,write.table)
importMethodsFrom(Biobase,'description<-')
importMethodsFrom(Biobase,description)
