#' GenomicInteractions
#'
#' Utilities for handling genomic interaction data such as
#' ChIA-PET or Hi-C, annotating genomic features with interaction
#' information, and producing plots and summary statistics.
#'
#' @name GenomicInteractions-package
#' @rdname GenomicInteractions-package
#' @docType package
#' @import methods
NULL
